Module: Deimos::KafkaListener
- Defined in:
- sig/defs.rbs
Overview
This module listens to events published by RubyKafka.
Class Method Summary collapse
-
.send_produce_error ⇒ void
Listens for any exceptions that happen during publishing and re-publishes as a Deimos event.
Class Method Details
.send_produce_error ⇒ void
This method returns an undefined value.
Listens for any exceptions that happen during publishing and re-publishes as a Deimos event.
@param event
663 |
# File 'sig/defs.rbs', line 663
def self.send_produce_error: (ActiveSupport::Notifications::Event event) -> void
|