Class: Deimos::Metrics::Provider
- Inherits:
-
Object
- Object
- Deimos::Metrics::Provider
- Defined in:
- lib/deimos/metrics/provider.rb,
sig/defs.rbs
Overview
Base class for all metrics providers.
Instance Method Summary collapse
-
#gauge(_metric_name, _count, _options = {}) ⇒ void
Send an counter increment metric.
-
#histogram(_metric_name, _count, _options = {}) ⇒ void
Send an counter increment metric.
-
#increment(_metric_name, _options = {}) ⇒ void
Send an counter increment metric.
-
#time(_metric_name, _options = {}) ⇒ void
Time a yielded block, and send a timer metric.
Instance Method Details
#gauge(_metric_name, _count, _options = {}) ⇒ void
This method returns an undefined value.
Send an counter increment metric
@param metric_name — The name of the counter metric
@param count
@param options — Any additional options, e.g. :tags
20 21 22 |
# File 'lib/deimos/metrics/provider.rb', line 20 def gauge(_metric_name, _count, ={}) raise MissingImplementationError end |
#histogram(_metric_name, _count, _options = {}) ⇒ void
This method returns an undefined value.
Send an counter increment metric
@param metric_name — The name of the counter metric
@param count
@param options — Any additional options, e.g. :tags
29 30 31 |
# File 'lib/deimos/metrics/provider.rb', line 29 def histogram(_metric_name, _count, ={}) raise MissingImplementationError end |
#increment(_metric_name, _options = {}) ⇒ void
This method returns an undefined value.
Send an counter increment metric
@param metric_name — The name of the counter metric
@param options — Any additional options, e.g. :tags
11 12 13 |
# File 'lib/deimos/metrics/provider.rb', line 11 def increment(_metric_name, ={}) raise MissingImplementationError end |
#time(_metric_name, _options = {}) ⇒ void
This method returns an undefined value.
Time a yielded block, and send a timer metric
@param metric_name — The name of the metric
@param options — Any additional options, e.g. :tags
37 38 39 |
# File 'lib/deimos/metrics/provider.rb', line 37 def time(_metric_name, ={}) raise MissingImplementationError end |