Module: Deimos::Utils::SchemaClass
- Defined in:
- lib/deimos/utils/schema_class.rb,
sig/defs.rbs
Overview
Class used by SchemaClassGenerator and Consumer/Producer interfaces
Class Method Summary collapse
-
.instance(payload, schema, namespace = '') ⇒ Deimos::SchemaClass::Record
Converts a raw payload into an instance of the Schema Class.
-
.klass(schema, namespace) ⇒ Deimos::SchemaClass
Determine and return the SchemaClass with the provided schema and namespace.
-
.modules_for(namespace) ⇒ ::Array[String]
@param
namespace. -
.use? ⇒ Boolean
@param
config— Producer or Consumer config.
Class Method Details
.instance(payload, schema, namespace = '') ⇒ Deimos::SchemaClass::Record
Converts a raw payload into an instance of the Schema Class
@param payload
@param schema
@param namespace
40 41 42 43 44 45 46 47 |
# File 'lib/deimos/utils/schema_class.rb', line 40 def instance(payload, schema, namespace='') return payload if payload.is_a?(Deimos::SchemaClass::Base) klass = klass(schema, namespace) return payload if klass.nil? || payload.nil? klass.(**payload.symbolize_keys) end |
.klass(schema, namespace) ⇒ Deimos::SchemaClass
Determine and return the SchemaClass with the provided schema and namespace
53 54 55 56 |
# File 'lib/deimos/utils/schema_class.rb', line 53 def klass(schema, namespace) constants = modules_for(namespace) + [schema.underscore.camelize.singularize] constants.join('::').safe_constantize end |
.modules_for(namespace) ⇒ ::Array[String]
@param namespace
11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 |
# File 'lib/deimos/utils/schema_class.rb', line 11 def modules_for(namespace) modules = ['Schemas'] namespace_override = nil module_namespace = namespace if Deimos.config.schema.use_full_namespace if Deimos.config.schema.schema_namespace_map.present? namespace_keys = Deimos.config.schema.schema_namespace_map.keys.sort_by { |k| -k.length } namespace_override = namespace_keys.find { |k| module_namespace.include?(k) } end if namespace_override.present? # override default module modules = Array(Deimos.config.schema.schema_namespace_map[namespace_override]) module_namespace = module_namespace.gsub(/#{namespace_override}\.?/, '') end namespace_folders = module_namespace.split('.').map { |f| f.underscore.camelize } modules.concat(namespace_folders) if namespace_folders.any? end modules end |
.use? ⇒ Boolean
@param config — Producer or Consumer config
957 |
# File 'sig/defs.rbs', line 957
def self.use?: (::Hash[untyped, untyped] config) -> bool
|