Module: Deimos::PhobosConfig
- Extended by:
- ActiveSupport::Concern
- Defined in:
- sig/defs.rbs
Overview
Module to handle phobos.yml as well as outputting the configuration to save to Phobos itself.
Instance Method Summary collapse
-
#phobos_config ⇒ ::Hash[untyped, untyped]
Create a hash representing the config that Phobos expects.
- #reset! ⇒ void
-
#ssl_var_contents ⇒ String
@param
key. - #to_h ⇒ ::Hash[untyped, untyped]
Instance Method Details
#phobos_config ⇒ ::Hash[untyped, untyped]
Create a hash representing the config that Phobos expects.
1224 |
# File 'sig/defs.rbs', line 1224
def phobos_config: () -> ::Hash[untyped, untyped]
|
#reset! ⇒ void
This method returns an undefined value.
1221 |
# File 'sig/defs.rbs', line 1221
def reset!: () -> void
|
#ssl_var_contents ⇒ String
@param key
1227 |
# File 'sig/defs.rbs', line 1227
def ssl_var_contents: (String key) -> String
|
#to_h ⇒ ::Hash[untyped, untyped]
1219 |
# File 'sig/defs.rbs', line 1219
def to_h: () -> ::Hash[untyped, untyped]
|