Class: Deimos::SchemaClass::Base
- Inherits:
-
Object
- Object
- Deimos::SchemaClass::Base
- Defined in:
- sig/defs.rbs
Overview
Base Class for Schema Classes generated from Avro.
Class Method Summary collapse
-
.initialize_from_value ⇒ SchemaClass::Base
Initializes this class from a given value.
Instance Method Summary collapse
-
#== ⇒ Boolean
@param
other. -
#[]= ⇒ void
@param
key. -
#as_json ⇒ ::Hash[untyped, untyped]
Converts the object to a hash which can be used for debugging or comparing objects.
- #hash ⇒ Integer
-
#initialize ⇒ Base
constructor
@param
_args. - #inspect ⇒ String
Constructor Details
#initialize ⇒ Base
@param _args
716 |
# File 'sig/defs.rbs', line 716
def initialize: (*::Array[Object] _args) -> void
|
Class Method Details
.initialize_from_value ⇒ SchemaClass::Base
Initializes this class from a given value
@param value
738 |
# File 'sig/defs.rbs', line 738
def self.initialize_from_value: (Object value) -> SchemaClass::Base
|
Instance Method Details
#== ⇒ Boolean
@param other
731 |
# File 'sig/defs.rbs', line 731
def ==: (SchemaClass::Base other) -> bool
|
#[]= ⇒ void
This method returns an undefined value.
@param key
@param val
728 |
# File 'sig/defs.rbs', line 728
def []=: ((String | Symbol) key, Object val) -> void
|
#as_json ⇒ ::Hash[untyped, untyped]
Converts the object to a hash which can be used for debugging or comparing objects.
@param _opts
@return — a hash representation of the payload
723 |
# File 'sig/defs.rbs', line 723
def as_json: (?::Hash[untyped, untyped] _opts) -> ::Hash[untyped, untyped]
|
#hash ⇒ Integer
740 |
# File 'sig/defs.rbs', line 740
def hash: () -> Integer
|
#inspect ⇒ String
733 |
# File 'sig/defs.rbs', line 733
def inspect: () -> String
|